import asyncio
from functools import wraps
from sys import version_info
from typing import List

import grpc
from rx import Observable

from treelab.rxmq_treelab.rxmq import Rxmq


def async_run(fn):
    """
    Wrapper that makes async run (the developers' lives as well) easier
    :param fn:
    :return:
    """

    @wraps(fn)
    def wrapped(*args, **kwargs):
        if version_info.major == 3:
            if version_info.minor >= 7:
                return asyncio.run(fn(*args, **kwargs))
            else:
                loop = asyncio.get_event_loop()
                return loop.run_until_complete(fn(*args, **kwargs))
        else:
            raise ValueError("Python 2 is not currently supported")

    return wrapped


def wait(event_name):
    """
    Wrapper for taking care of asynchronous grpc calls
    """

    def decorator(fn):
        @wraps(fn)
        def wrapper(*args, workspace_id: str = None, wait_till_complete: bool = True, name_spaces: List[str] = None):
            if not workspace_id and event_name != 'WorkspaceCreated':
                raise ValueError('workspace_id can be None only under the case when event_name == WorkspaceCreated')
            if not name_spaces:
                name_spaces = []
            grpc_api, grpc_input, meta_data = fn(*args)
            if wait_till_complete:
                response = grpc_api(grpc_input, metadata=meta_data)
                complete_event_name = '.'.join(name_spaces + [response.id, event_name])
                if event_name != 'WorkspaceCreated':
                    wait_for_first_event(workspace_id, complete_event_name)
                return response
            else:
                future: grpc.Future = grpc_api.future(grpc_input)
                return future

        return wrapper

    return decorator


def wait_for_first_event(workspace_id: str, event_name: str) -> Observable:
    """
    Listening for the first event with topic under workspace with workspace_id
    :param workspace_id:
    :param event_name:
    :return:
    """
    return Rxmq.channel(workspace_id).observe(event_name)



# server
workspace_id = 'wsp46bf41a21d09c07f'
# core 1
core_id_1 = 'cre488fee3799074eee'
view_id_1 = 'viw488ff6c68905750d'
table_id_1 = 'tbl488ff16957075d9d'
column_id_1 = 'col48907ad50d8b487b'
column_name_1 = 'col_name1'
view_name_1 = 'viewname'
row_id_1 = 'row489068979504c18c'
core_name_1 = 'test_pysdk'

# # local
# workspace_id = 'wspb59083e8e085e9af'
# # core 1
# core_id_1 = 'creb59083ec830851cc'
# view_id_1 = 'viwb59083f069813b22'
# table_id_1 = 'tblb59083ee9505054d'
# column_id_1 = 'colb59084087b8259b7'
# column_name_1 = 'col_name1'
# view_name_1 = 'viewname'
# row_id_1 = 'rowb590840785034627'



from treelab.grpc_treelab.messages.service_pb2 import EventPayload


def get_event_identifier(event: EventPayload):
    topic = event.eventName.split('.')[-1]
    # prefixes = []
    # if topic == 'CoreCreated':
    #     return event.eventName
    # if topic == 'TableCreated':
    #     prefixes.append(event.tableId)
    # elif topic == 'RowAddedToView':
    #     prefixes.append(event.rowId)
    # elif topic == 'ColumnAddedToView':
    #     prefixes.append(event.columnId)
    # elif topic == 'CellUpdated':
    #     prefixes.extend([event.rowId + event.columnId, event.tableId])
    # elif topic == 'ViewAdded':
    #     prefixes.append(event.view.id)
    #
    # return '.'.join(prefixes + [topic])
    if topic == 'CoreCreated':
        return event.eventName
    if topic == 'TableCreated':
        return event.eventName
    elif topic == 'RowAddedToView':
        return event.eventName
    elif topic == 'ColumnAddedToView':
        return event.eventName
    elif topic == 'CellUpdated':
        return event.eventName
    elif topic == 'ViewAdded':
        return event.eventName
